/**
    EncryptedDll.c - Read an encrypted DLL from disk and decrypt it using a key loaded from a SmartDongle

    MicroWorks, Inc.
    2808 North Cole Road
    Boise, ID 83704

    www.smartdongle.com


    ***** BEGIN LICENSE BLOCK *****

    Version: MPL 1.1/GPL 2.0/LGPL 2.1

    The contents of this file are subject to the Mozilla Public License
    Version 1.1 (the "License"); you may not use this file except in
    compliance with the License. You may obtain a copy of the License at
    http://www.mozilla.org/MPL/

    Software distributed under the License is distributed on an "AS IS" basis,
    WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
    for the specific language governing rights and limitations under the
    License.

    The Original Code is MicroWorks, Inc. code.

    The Initial Developer of the Original Code is MicroWorks, Inc. Portions
    created by MicroWorks, Inc. are Copyright (C) 2010
    MicroWorks, Inc. All Rights Reserved.

    Contributor(s):

    Alternatively, the contents of this file may be used under the terms of
    either the GNU General Public License Version 2 or later (the "GPL"), or
    the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
    in which case the provisions of the GPL or the LGPL are applicable instead
    of those above. If you wish to allow use of your version of this file only
    under the terms of either the GPL or the LGPL, and not to allow others to
    use your version of this file under the terms of the MPL, indicate your
    decision by deleting the provisions above and replace them with the notice
    and other provisions required by the GPL or the LGPL. If you do not delete
    the provisions above, a recipient may use your version of this file under
    the terms of any one of the MPL, the GPL or the LGPL.

    ***** END LICENSE BLOCK *****
*/

#include <tchar.h>
#include <Windows.h>
#include <stdio.h>
#include "MemoryModule.h"
#include "smartdongle.h"
#include "sd_helper.h"
#include "sd_uskerr.h";
#include "aes.h";

/* function pointer to the factorial function in the MyDll project */
typedef int (__cdecl *PFACTORIAL)(int);

/* size of the AES key */
#define KEY_SIZE 16

/* 128 bit block encryption */
#define AES_BLOCKSIZE 128    


int _tmain(int argc, _TCHAR* argv[])
{
		/* Your SmartDongle keys */
	unsigned __int64 P1 = 0xec6cc589aefd1e75;
	unsigned __int64 P2 = 0xfcec0a6a82747b3f;



	JIKpbm1 *data = NULL;
	long fsize = 0;
	size_t amountRead = 0;
	char* encData = NULL;

	/* address to write the key at */
	unsigned short address = 0;    
    char errorString[QTS_ERROR_STRING_LENGTH_MAX];
    int error = USK_OK;

	int val = 0;
	
	PFACTORIAL pFact = NULL;
	HMEMORYMODULE dll = NULL;

	/* allocate the size for the key plus 1 byte for NULL terminator */
	unsigned char* key = (unsigned char*)malloc(KEY_SIZE +1);
	if (key == NULL) {
        error = USK_ERR_MEM;
        SmartDongleGetErrorString(errorString, error);
        printf("%s\n", errorString);
	}

    /*
        Read the key from the dongle
    */
    if (!error) {
        error = SmartDongleRead(P1, P2, address, key, KEY_SIZE);
		key[KEY_SIZE] = NULL;
        if (error) {
            SmartDongleGetErrorString(errorString, error);
            printf("%s\n", errorString);
        } else {
			/* decrypt the file */
			FILE *fp = fopen("MyDll_e.dll", "rb");
			if (fp == NULL)
			{
				printf("Error opening encrypted DLL!\n");
				return 0;
			}
			fseek(fp, 0L, SEEK_END);
			fsize = ftell(fp);
			fseek(fp, 0L, SEEK_SET);
			encData = (char*)malloc(fsize);
			ZeroMemory(encData, fsize);
			amountRead = fread(encData, sizeof(char),fsize, fp);
			fclose(fp);			
			JIKpbm3(
					(unsigned char*)encData,
					fsize / (AES_BLOCKSIZE / 8),
					key,
					AES_BLOCKSIZE);											
        }
    }

	/* load the dll from memory */
	dll = MemoryLoadLibrary((void*)encData);

	if (dll == NULL)
	{
		printf("Error loading DLL from memory!");
		return 0;
	}

	/* set up the function pointer to the "factorial" function */
	pFact = (PFACTORIAL)MemoryGetProcAddress(dll, "factorial");

	val = pFact(7);

	printf_s("%d\n\n", val);

	/* free the library when we are done */
	MemoryFreeLibrary(dll);

	return 0;
}

