/*

    example3a.c:  Example that encrypts a user string and
                  writes the encrypted string to SmartDongle.

    www.smartdongle.com

    MicroWorks, Inc.
    2808 North Cole Road
    Boise, ID 83704
*/

#include <stdio.h>
#include <malloc.h>
#include "smartdongle.h"
#include "sd_eeprom.h"
#include "sd_uskerr.h"
#include "aes.h"
#include <conio.h>



/*
    For demonstration only, use your own unique AES key and secrets for
    your application.
*/

/* Your secret */
//#define YOUR_SECRET "This is the secret written to SmartDongle, it can be passwords, serial numbers, program parameters, etc"
#define YOUR_SECRET "=EC6CC589AEFD1E75FCEC0A6A82747B3FEC6CC589AEFD1E75FCEC0A6A82747B3FEC6CC589AEFD1E75FCEC0A6A82747B3FEC6CC589AEFD1E75FCEC0A6A82747B3F"

/* Some random 128 bit AES key */
//#define AES_KEY "7obdgEn2'TH>]SE|" 
#define AES_KEY "RyAn'SEncrYptioN" 

/* 128 bit block encryption */
#define AES_BLOCKSIZE 128    

int main(int argc, char* argv[])
{
    unsigned __int64 P1, P2;
    unsigned char key[] = AES_KEY;
    unsigned char your_secret[] = YOUR_SECRET;
    char errorString[QTS_ERROR_STRING_LENGTH_MAX];
    int size = AES_BLOCKSIZE;
    int error = USK_OK;
    JIKpbm1 *data = NULL;
	FILE *fp;
    /*
        For data integrity check.
        The number of blocks your secret requires and address in SmartDongle
        memory will always be known value to the developer. These parmeters
        are printed out to the console below with each successfull operation.
		cipherTextBlocks = strlen((char*)your_secret)*8/AES_BLOCKSIZE;
    */
    const unsigned cipherTextBlocks = 9;
	unsigned char cipherText[9 * (AES_BLOCKSIZE / 8)];
    const unsigned short cipherTextAddress = 0;


    /*
        Your SmartDongle key go here.
    */
#ifdef _GNUC_ /* For GNU  */
    P1 = 0xec6cc589aefd1e75ULL;
    P2 = 0xfcec0a6a82747b3fULL;
#else
    P1 = 0xec6cc589aefd1e75;
    P2 = 0xfcec0a6a82747b3f;
#endif


    /*
        Turn SmartDongle Red LED on to indicate write in progress.
    */
    SmartDongleLedRed();


    /*
        Encrypt your secret here using this function:

        JIKpbm1* JIKpbm2(
            uint8 *scrt,
            uint32 scrtLen,
            uint8 *ky,
            uint32 blockLen);


        Parameters:

        scrt
            Pointer to character string containing your passwords,
            serial numbers, etc.

        scrtLen
            Length of scrt string above, excluding the terminal NULL.

        ky
            Your 128 bit encryption key (some 16 byte character string).

        blockLen
            The block length used for the AES algorithm, in bits.
            This value should be set to 128 bits.


        Returns:
            A struct containing encrypted user data and the total number
            of blocks used.         
    */
    data = JIKpbm2(
        your_secret,
        strlen((char *)your_secret),
        key,
        AES_BLOCKSIZE);
    if (data == NULL) {
        error = USK_ERR_MEM;
        SmartDongleGetErrorString(errorString, error);
        printf("%s\n", errorString);
    }

	fp = fopen("test.txt", "w");
	fprintf(fp, (char*)data->str);
	fclose(fp);
    /*
        Write your encrypted secret to SmartDongle memory.
    */
    if (!error) {
        error = SmartDongleWrite(
            P1,
            P2,
            cipherTextAddress,
            data->str,
            data->numberOfBlocks * (AES_BLOCKSIZE / 8));

        if (error) {
            SmartDongleGetErrorString(errorString, error);
            printf("%s\n", errorString);
        }
    }


    /*
        Data integrity check.
      Read encrypted secret back from SmartDongle's user memory and decrypt.
    */
    if (!error) {
        error = SmartDongleRead(
            P1,
            P2,
            cipherTextAddress,
            cipherText,
            cipherTextBlocks * (AES_BLOCKSIZE / 8));

        if (error) {
            SmartDongleGetErrorString(errorString, error);
            printf("%s\n", errorString);
        }
    }
	
    // Decrypt and compare to expected user data
    if (!error) {

        JIKpbm3(cipherText, cipherTextBlocks, key, AES_BLOCKSIZE);
        if(strcmp(cipherText, your_secret) != 0) {
            printf("Failure: decoded data does not match your secret\n");
            error = USK_ERR_UNKNOWN;
        }
    }


    if (!error) {

        printf("Success: Wrote and verified your secret to SmartDongle\n");
        printf("\n");
        printf("Use these parameters in example3b.c\n");
        printf("AES Block Size (in bits): %d\n", AES_BLOCKSIZE);
        printf("Number of blocks used: %d\n", data->numberOfBlocks);
        printf("cipherTextAddress: %d\n", cipherTextAddress);
 
        SmartDongleLedGreen();
    }


    printf("\nPress any key to exit\n");
    getch();

    if (data != NULL) {
        if (data->str != NULL) {
            free(data->str);
        }
        free(data);
    }

    return 0;
}



