/*
   sd_helper.h for SmartDongle API version 3.2


   www.smartdongle.com

   MicroWorks, Inc.
   2808 North Cole Road
   Boise, ID 83704
*/
#ifndef _DEFINE_D4011684_73C2_4b6c_9799_7CE60F023647
#define _DEFINE_D4011684_73C2_4b6c_9799_7CE60F023647

#if defined(__cplusplus) || defined(__cplusplus__)
extern "C" {
#endif

typedef struct smartdongleDrvVer smartdongleDrvVer_t;

/*
   Driver version x.x.x.x
*/
struct smartdongleDrvVer
{
   unsigned char m3;
   unsigned char m2;
   unsigned char minor;
   unsigned char major;
};

/* IOCTL parameters */
#define QTS_IOCTL_RETRYS 5
#define QTS_IOCTL_FAIL_WAIT 50
#define QTS_IOCTL_BUFFSIZE 1024

/* Reset delays */
#define QTS_READY_RETRIES 8
#define QTS_READY_WAIT 250
#define QTS_PLUG_N_PLAY_WAIT 1000

#define QTS_MAXPREFETCH (CP2_PAGE_PREFETCH_ENABLED*EEPROMPAGESIZE)

/*
   Three on-chip SmartDongle encrpytion modes to choose from.
   This provide protection for SmartDongle on-board memory only.
   Select QTS_OPTION_CRYPT_IO to encrypt data written to SmartDongle
   user memory.
*/
#define QTS_OPTION_ON_CHIP_EPROM_PROTECT_STRONG   0x00
#define QTS_OPTION_ON_CHIP_EPROM_PROTECT_DISABLE  0x01
#define QTS_OPTION_ON_CHIP_EPROM_PROTECT_FAST     0x02

/*
   Old API
*/
#define QTSSTRONGCRYPT QTS_OPTION_ON_CHIP_EPROM_PROTECT_STRONG
#define QTSCRYPTDISABLED QTS_OPTION_ON_CHIP_EPROM_PROTECT_DISABLE
#define QTSFASTCRYPT QTS_OPTION_ON_CHIP_EPROM_PROTECT_FAST

/*
   Fixed options on SmartDongle queried by getOpt()
*/
/* SmartDongle v1.70 or above, extended functionality */
#define QTS_OPTION_EXTENDED_SUPPORT2  0x20

/* SmartDongle requires a new key for each access */
#define QTS_OPTION_STRICT_ONLY       0x40

/* SmartDongle v1.50 or above, extended functionality */
#define QTS_OPTION_EXTENDED_SUPPORT  0x80

/*
  Open handle to the first SmartDongle found - not for multiple dongles.
*/
int OpenUsbDev (HANDLE *m_UsbHandle);

/*
   SmartDongle Unlock/Lock
*/
int GetSmartDongleRevision(char* revBuffer);
int EnableUsk (unsigned __int64 P1, unsigned __int64 P2, HANDLE *m_UsbHandle);
unsigned __int64 getKeylessChallenge(HANDLE *m_UsbHandle);
int sendKeylessResponse(HANDLE *m_UsbHandle, unsigned __int64 response);
int KeylessReadEepromUserSpace(
   unsigned short address,
   unsigned char *buffer,
   int *size,
   HANDLE *m_UsbHandle);
int KeylessWriteEepromUserSpace(
   unsigned short address,
   unsigned char *buffer,
   int *size,
   HANDLE *m_UsbHandle);
int DisableUsk (HANDLE *m_UsbHandle);

/*
   Current Security State
*/
int readSecurityState (unsigned char *state, HANDLE *m_UsbHandle);

/*
  Read/Write to user memory on an unlocked SmartDongle.
*/
int ReadEepromUserSpace (
   unsigned __int64 P1,
   unsigned __int64 P2,
   unsigned short address,
   unsigned char *buffer,
   int *size,
   HANDLE *m_UsbHandle);

int WriteEepromUserSpace (
   unsigned __int64 P1,
   unsigned __int64 P2,
   unsigned short address,
   unsigned char *buffer,
   int *size,
   HANDLE *m_UsbHandle);

/*
  Reboot a SmartDongle.
*/
int SoftReset (HANDLE *m_UsbHandle);

/*
  Strong encryption/decryption is slow, you may want to use a fast but less
  secure algorithm, or just read/write to the SmartDongle user memory in
  plain text (no encryption).

  Set sequential eprom read flag to dump eprom contents without issuing
  a page address for each fetch.

  Mode byte bitmap:
    0x00  strong encryption
    0x01  encryption disabled.
    0x02  fast encryption.
*/
int SetMode (
   unsigned __int64 P1,
   unsigned __int64 P2,
   unsigned char mode,
   HANDLE *m_UsbHandle);

int GetMode (unsigned char *mode, HANDLE *m_UsbHandle);

/*
  Reading the SmartDongle serial number may be useful.
*/
int getSerialNumber (char *serNo, HANDLE *m_UsbHandle);

int setStrictOnly (HANDLE *m_UsbHandle);

/*
  LED control
*/
int Green (HANDLE *m_UsbHandle);
int Red (HANDLE *m_UsbHandle);
int Off (HANDLE *m_UsbHandle);

int waitForBusy (unsigned int period, HANDLE *m_UsbHandle);

int ready(DWORD dwMilliseconds, HANDLE *m_UsbHandle);


#if defined(__cplusplus) || defined(__cplusplus__)
}
#endif

#endif /* _DEFINE_D4011684_73C2_4b6c_9799_7CE60F023647 */
